/*
 * Copyright (c) 2002-2008 Hypertriton, Inc. <http://hypertriton.com/>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <config/have_vasprintf.h>
#include <config/_mk_have_sys_types_h.h>

#if defined(__linux__) && !defined(_GNU_SOURCE)
#define _GNU_SOURCE
#endif

#ifdef _MK_HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "core.h"

int
AG_Vasprintf(char **ret, const char *fmt, va_list ap)
{
#ifndef HAVE_VASPRINTF
	char *buf;
	int size;
	size_t buflen;

	buflen = strlen(fmt) + 10240;			/* XXX */
	if ((buf = TryMalloc(buflen)) == NULL) {
		return (-1);
	}
	size = vsprintf(buf, fmt, ap);
	if ((size_t)size <= buflen) {
		*ret = buf;
		return (size);
	}

	if ((buf = TryRealloc(buf, size+1)) == NULL) {
		Free(buf);
		return (-1);
	}
	size = vsprintf(buf, fmt, ap);
	*ret = buf;
	return (size);
#else /* !HAVE_VASPRINTF */
	if (vasprintf(ret, fmt, ap) == -1) {
		AG_SetError("Out of memory");
		return (-1);
	}
	return (0);
#endif /* HAVE_VASPRINTF */
}
